import sys
import requests
import time
import json
import os
import clickhouse_connect
import pandas as pd
from datetime import datetime, timedelta
from io import StringIO
import zipfile
import io
from .common import Common

class OZONreklama:
    def __init__(self, bot_token:str, chat_list:str, message_type: str, subd: str, add_name: str, clientid:str, token: str , host: str, port: str, username: str, password: str, database: str, start: str, backfill_days: int):
        self.bot_token = bot_token
        self.chat_list = chat_list
        self.message_type = message_type
        self.clientid = clientid
        self.token = token
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.add_name = add_name.replace(' ','').replace('-','_')
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.yesterday = self.today - timedelta(days = 1)
        self.start = start
        self.backfill_days = backfill_days
        self.err429 = False
        self.client = clickhouse_connect.get_client(host=host, port=port, username=username, password=password, database=database)


    def ch_insert(self, df, to_table):
        data_tuples = [tuple(x) for x in df.to_numpy()]
        self.client.insert(to_table, data_tuples, column_names=df.columns.tolist())



    def chunk_list(self, lst, chunk_size):
        for i in range(0, len(lst), chunk_size):
            yield lst[i:i + chunk_size]

    def convert_to_timestamp(self, datetime_str):
        # Используем pandas для преобразования строки в timestamp
        timestamp = pd.to_datetime(datetime_str, utc=True).timestamp()
        # Округляем до целого числа для совместимости с ClickHouse DateTime
        return int(timestamp)

    def get_token(self, client_id, client_secret):
        try:
            host = 'https://api-performance.ozon.ru'
            endpoint = '/api/client/token'
            headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
            payload = {"client_id": client_id, "client_secret": client_secret, "grant_type": "client_credentials"}
            res = requests.post(host + endpoint, headers=headers, json=payload)
            if res.status_code == 200:
                message =  f"Платформа: OZON_ADS. Имя: {self.add_name}. Токен получен успешно"
                self.common.log_func(self.bot_token, self.chat_list, message, 1)
            else:
                message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Ошибка получения токена: {res.status_code} {res.text}"
                self.common.log_func(self.bot_token, self.chat_list, message, 3)
            response_json = res.json()
            if 'access_token' in response_json:
                access_token = response_json['access_token']
            else:
                access_token=None
            return access_token
        except Exception as e:
            message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Ошибка: {str(e)}"
            self.common.log_func(self.bot_token, self.chat_list, message, 3)


    def get_names(self, token):
        try:
            url = 'https://api-performance.ozon.ru:443/api/client/campaign'
            headers = {'Authorization': f'Bearer {token}','Content-Type': 'application/json','Accept': 'application/json'}
            response = requests.get(url, headers=headers)
            try:
                result = response.json()
            except:
                result = None
            message = f'Платформа: OZON_ADS. Имя: {self.add_name}. Функция: get_names. Код: {str(response.status_code)}'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            if response.status_code != 200:
                message = f'Платформа: OZON_ADS. Имя: {self.add_name}. Функция: get_names. Результат: {str(result)}'
                self.common.log_func(self.bot_token, self.chat_list, message, 1)
            if response.status_code == 200 and result != None:
                campaigns = response.json()['list']
                df = pd.json_normalize(campaigns)
                pd.set_option('display.max_columns', None)
                df.rename(columns={'autopilot.maxBid': 'autopilot_maxBid'}, inplace=True)
                df.rename(columns={'autopilot.categoryId': 'autopilot_categoryId'}, inplace=True)
                df.rename(columns={'autopilot.filters': 'autopilot_filters'}, inplace=True)
                df.rename(columns={'autopilot.skuAddMode': 'autopilot_skuAddMode'}, inplace=True)
                df['createdAt'] = df['createdAt'].apply(self.convert_to_timestamp)
                df['updatedAt'] = df['updatedAt'].apply(self.convert_to_timestamp)
                df['timeStamp'] = self.now
                df['fromDate'] = pd.to_datetime(df['fromDate'], errors='coerce')
                df['toDate'] = pd.to_datetime(df['toDate'], errors='coerce')
                df['toDate'].fillna(pd.Timestamp.today().normalize(), inplace=True)
                df['fromDate'].fillna(pd.Timestamp.today().normalize(), inplace=True)
                df['id'] = df['id'].astype('int64')
                df['placement'] = df['placement'].astype('str')
                df['autopilot_filters'] = df['autopilot_filters'].astype('str')
                required_columns = ['id', 'title', 'state', 'advObjectType', 'fromDate', 'toDate',
                        'dailyBudget', 'placement', 'budget', 'createdAt', 'updatedAt',
                        'productCampaignMode', 'productAutopilotStrategy', 'PaymentType',
                        'expenseStrategy', 'weeklyBudget', 'budgetType', 'startWeekDay',
                        'endWeekDay', 'autopilot_maxBid', 'autopilot_categoryId',
                        'autopilot_skuAddMode', 'autopilot_filters', 'timeStamp']
                df = df[required_columns]
                self.ch_insert(df, f"ozon_ads_campaigns_{self.add_name}")
                return response.status_code
        except Exception as e:
            message = f'Платформа: OZON_ADS. Имя: {self.add_name}. Функция: get_names. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return None

    def text_to_df(self, response_text, date):
        pd.set_option('display.max_columns', None)
        csv_file = response_text.splitlines()
        sp_columns = ['orderDate', 'orderId', 'orderNum', 'ozonId', 'productOzonId', 'artikul', 'name', 'count', 'price', 'value', 'rate', 'cost', ]
        sku_columns = ['sku', 'productName', 'productPrice', 'views', 'clicks', 'cost', 'orders', 'revenue',  'modelOrders', 'modelRevenue']
        banner_columns = ['banner', 'pageType', 'viewCond', 'platform', 'views', 'clicks', 'reach', 'cost']
        brand_shelf_columns = ['conditionType', 'viewCond', 'platform', 'views', 'clicks', 'reach', 'cost']
        sis_columns = ['pageType', 'views', 'clicks', 'cost', 'reach']

        replace_dict = {'Название товара': 'productName', 'Цена товара, ₽': 'productPrice', 'Расход, ₽, с НДС': 'cost',
                        'Показы': 'views', 'Заказы': 'orders', 'Клики': 'clicks', 'Выручка, ₽': 'revenue',
                        'Заказы модели': 'modelOrders', 'Выручка с заказов модели, ₽': 'modelRevenue',
                        'Тип страницы': 'pageType', 'Охват': 'reach', 'Тип условия': 'conditionType', 'Условие показа': 'viewCond',
                        'Платформа': 'platform', 'Баннер': 'banner', 'Дата': 'orderDate', 'ID заказа': 'orderId', 'Номер заказа': 'orderNum',
                        'Ozon ID': 'ozonId','Ozon ID продвигаемого товара': 'productOzonId', 'Артикул': 'artikul', 'Наименование': 'name',
                        'Количество': 'count', 'Цена продажи': 'price', 'Стоимость, ₽': 'value', 'Ставка, ₽': 'rate', 'Расход, ₽': 'cost'}

        int_list = ['views', 'clicks', 'orders', 'modelOrders', 'reach', 'count']
        float_list = ['cost', 'revenue', 'modelRevenue', 'productPrice', 'price', 'value', 'rate']
        str_list = ['orderNum', 'orderDate', 'productName', 'pageType', 'conditionType', 'viewCond', 'platform', 'banner', 'orderId', 'ozonId', 'productOzonId', 'artikul', 'name']


        add_to_table = "unknown"
        if len(csv_file)>1:
            csv_data = '\n'.join(csv_file[1:])
            campaign_id = int(csv_file[0].split('№')[1].split(',')[0].strip())
            date_as_date = datetime.strptime(date, '%Y-%m-%d')
            df = pd.read_csv(StringIO(csv_data), sep=';')
            first_column_name = df.columns[0]

            df_filtered = df.query(f'`{first_column_name}` != "Корректировка" and `{first_column_name}` != "Всего"')
            for key, value in replace_dict.items():
                try:
                    df_filtered = df_filtered.rename(columns={key: value})
                except:
                    pass

            if set(sp_columns).issubset(df_filtered.columns):
                add_to_table = 'sp'
                df_filtered = df_filtered[sp_columns]
            elif set(sku_columns).issubset(df_filtered.columns):
                add_to_table = 'sku'
                df_filtered = df_filtered[sku_columns]
            elif set(banner_columns).issubset(df_filtered.columns):
                add_to_table = 'banner'
                df_filtered = df_filtered[banner_columns]
            elif set(brand_shelf_columns).issubset(df_filtered.columns):
                add_to_table = 'shelf'
                df_filtered = df_filtered[brand_shelf_columns]
            elif set(sis_columns).issubset(df_filtered.columns):
                add_to_table = 'sis'
                df_filtered = df_filtered[sis_columns]


            df_filtered['date'] = date_as_date
            df_filtered['id'] = campaign_id
            df_filtered['timeStamp'] = self.now

            for col in int_list:
                try: df_filtered[col] = df_filtered[col].astype('int64')
                except: pass
            for col in float_list:
                try: df_filtered[col] = df_filtered[col].astype(str).str.replace(',', '.')
                except: pass
            for col in str_list:
                try: df_filtered[col] = df_filtered[col].astype(str)
                except: pass


            return [df_filtered, add_to_table]
        else:
            empty_df = pd.DataFrame()
            return [empty_df, add_to_table]

    def get_data(self, token, campaigns, date):
        if self.err429 == False:
            try:
                url = 'https://api-performance.ozon.ru:443/api/client/statistics'
                headers = {'Authorization': f'Bearer {token}'}
                payload = {
                    "campaigns": campaigns,
                    "dateFrom": date,
                    "dateTo": date,
                    "groupBy": "NO_GROUP_BY"
                }
                response = requests.post(url, headers=headers, json=payload)
                if response.status_code != 200:
                    message = f'Платформа: OZON_ADS. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_data. Результат: {str(response.status_code)} {str(response.json())}'
                    self.common.log_func(self.bot_token, self.chat_list, message, 2)
                else:
                    report_uuid = response.json()['UUID']
                    url = f'https://api-performance.ozon.ru:443/api/client/statistics/{report_uuid}'
                    for k in range(300):
                        time.sleep(5)
                        try:
                            response = requests.get(url, headers=headers)
                            if response.json()['state']=="OK":
                                break
                        except:
                            pass
                    url = f'https://api-performance.ozon.ru:443/api/client/statistics/report?UUID={report_uuid}'
                    headers = {'Authorization': f'Bearer {token}'}
                    response = requests.get(url, headers=headers)
                    if len(campaigns) == 1:
                        text_df = self.text_to_df(response.text, date)
                        add_to_table = text_df[1]
                        df = text_df[0]
                        self.ch_insert(df, f"ozon_ads_data_{add_to_table}_{self.add_name}")
                    else:
                        with zipfile.ZipFile(io.BytesIO(response.content)) as zip_file:
                            num = 0
                            for file_name in zip_file.namelist():
                                num +=1
                                with zip_file.open(file_name) as file:
                                    content = file.read().decode('utf-8')
                                    text_df = self.text_to_df(content, date)
                                    add_to_table = text_df[1]
                                    df = text_df[0]
                                    num_rows = df.shape[0]
                                    if num_rows>0:
                                        self.ch_insert(df, f"ozon_ads_data_{add_to_table}_{self.add_name}")
                                time.sleep(2)
                return response.status_code
            except Exception as e:
                message= f'Платформа: OZON_ADS. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_data. Ошибка: {e}'
                self.common.log_func(self.bot_token, self.chat_list, message, 3)
                return None
        else:
            message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_data. Ошибка 429, запрос не отправлен."
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return None

    def get_campaigns_in_period(self, token, start_date):
        try:
            end_date = self.yesterday.strftime("%Y-%m-%d")
            url = 'https://api-performance.ozon.ru:443/api/client/campaign'
            headers = {'Authorization': f'Bearer {token}','Content-Type': 'application/json','Accept': 'application/json'}
            response = requests.get(url, headers=headers)
            try:
                result = response.json()
            except:
                result = None
            message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Функция: get_campaigns_in_period. Код: {str(response.status_code)}"
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            if response.status_code != 200:
                message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Функция: get_campaigns_in_period. Ответ: {str(result)}"
                self.common.log_func(self.bot_token, self.chat_list, message, 1)
            if response.status_code == 200 and result != None:
                campaigns = response.json()['list']
                df = pd.json_normalize(campaigns)
                pd.set_option('display.max_columns', None)
                df['createdAt'] = df['createdAt'].str[:10]
                df['toDate'] = pd.to_datetime(df['toDate'], errors='coerce')
                df['toDate'].fillna(pd.Timestamp.today().normalize(), inplace=True)
                df['toDate'] = df['toDate'].dt.strftime('%Y-%m-%d')
                df['id'] = df['id'].astype('int64')
                required_columns = ['id', 'createdAt', 'toDate']
                df = df[required_columns]
                df_filtered = df[((df['createdAt'] <= end_date) & (df['toDate'] >= start_date))
                                 | ((df['toDate'] >= start_date) & (df['createdAt'] <= end_date))
                                 | ((df['createdAt'] >= start_date) & (df['toDate'] <= end_date))
                                 | ((df['createdAt'] <= start_date) & (df['toDate'] >= end_date))]
                advert_id_list = df_filtered['id'].tolist()
                return advert_id_list
        except Exception as e:
            message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Функция: get_campaigns_in_period. Ошибка: {e}"
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return None

    def create_date_list(self, start_date_str, end_date_str):
        try:
            # Преобразование строк в объекты datetime
            start_date = datetime.strptime(start_date_str, '%Y-%m-%d')
            end_date = datetime.strptime(end_date_str, '%Y-%m-%d')

            # Генерация списка дат
            date_list = []
            current_date = start_date
            while current_date <= end_date:
                date_list.append(current_date.strftime('%Y-%m-%d'))
                current_date += timedelta(days=1)

            return date_list
        except Exception as e:
            message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Функция: create_date_list. Ошибка: {e}"
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return []


    def ozon_reklama_collector(self):
        optimize_data_sp = f"OPTIMIZE TABLE ozon_ads_data_sp_{self.add_name} FINAL"
        optimize_data_sku = f"OPTIMIZE TABLE ozon_ads_data_sku_{self.add_name} FINAL"
        optimize_data_banner = f"OPTIMIZE TABLE ozon_ads_data_banner_{self.add_name} FINAL"
        optimize_data_shelf = f"OPTIMIZE TABLE ozon_ads_data_shelf_{self.add_name} FINAL"
        optimize_data_sis = f"OPTIMIZE TABLE ozon_ads_data_sis_{self.add_name} FINAL"
        optimize_campaigns = f"OPTIMIZE TABLE ozon_ads_campaigns_{self.add_name} FINAL"
        optimize_collection = f"OPTIMIZE TABLE ozon_ads_collection_{self.add_name} FINAL"


        create_table_query_campaigns = f"""
        CREATE TABLE IF NOT EXISTS ozon_ads_campaigns_{self.add_name} (
            id UInt64, 
            title String, 
            state String, 
            advObjectType String, 
            fromDate Date, 
            toDate Date,
            dailyBudget Float, 
            placement String, 
            budget Float, 
            createdAt DateTime, 
            updatedAt DateTime,
            productCampaignMode String, 
            productAutopilotStrategy String, 
            PaymentType String,
            expenseStrategy String, 
            weeklyBudget Float, 
            budgetType String, 
            startWeekDay String,
            endWeekDay String, 
            autopilot_maxBid Float, 
            autopilot_categoryId UInt64,
            autopilot_skuAddMode String, 
            autopilot_filters String,
            timeStamp DateTime
        ) ENGINE = ReplacingMergeTree(timeStamp)
        ORDER BY id
        """

        create_table_query_data_sp = f"""
        CREATE TABLE IF NOT EXISTS ozon_ads_data_sp_{self.add_name} (
            date Date,
            id UInt64, 
            orderDate String,
            orderId String,
            orderNum String, 
            ozonId String, 
            productOzonId String, 
            artikul String, 
            name String,
            count UInt64, 
            price Float, 
            value Float, 
            rate Float, 
            cost Float,
            timeStamp DateTime
        ) ENGINE = ReplacingMergeTree(timeStamp)
        ORDER BY (date,id,orderDate,orderId,ozonId,productOzonId,artikul)
        """

        create_table_query_data_sku = f"""
        CREATE TABLE IF NOT EXISTS ozon_ads_data_sku_{self.add_name} (
            date Date,
            id UInt64, 
            sku String,
            productName String, 
            productPrice Float, 
            views UInt64, 
            clicks UInt64, 
            cost Float,
            orders UInt64, 
            revenue Float, 
            modelOrders UInt64, 
            modelRevenue Float, 
            timeStamp DateTime
        ) ENGINE = ReplacingMergeTree(timeStamp)
        ORDER BY (date,id,sku)
        """

        create_table_query_data_banner = f"""
            CREATE TABLE IF NOT EXISTS ozon_ads_data_banner_{self.add_name} (
                date Date,
                id UInt64, 
                banner String,
                pageType String, 
                viewCond String, 
                platform String, 
                views UInt64, 
                clicks UInt64, 
                cost Float,
                reach UInt64,  
                timeStamp DateTime
            ) ENGINE = ReplacingMergeTree(timeStamp)
            ORDER BY (date,id,banner,pageType,viewCond, platform)
            """

        create_table_query_data_shelf = f"""
                CREATE TABLE IF NOT EXISTS ozon_ads_data_shelf_{self.add_name} (
                    date Date,
                    id UInt64, 
                    conditionType String, 
                    viewCond String, 
                    platform String, 
                    views UInt64, 
                    clicks UInt64, 
                    cost Float,
                    reach UInt64,  
                    timeStamp DateTime
                ) ENGINE = ReplacingMergeTree(timeStamp)
                ORDER BY (date,id,conditionType,viewCond, platform)
            """

        create_table_query_data_sis = f"""
                CREATE TABLE IF NOT EXISTS ozon_ads_data_sis_{self.add_name} (
                    date Date,
                    id UInt64, 
                    pageType String, 
                    views UInt64, 
                    clicks UInt64, 
                    cost Float,
                    reach UInt64,  
                    timeStamp DateTime
                ) ENGINE = ReplacingMergeTree(timeStamp)
                ORDER BY (date,id,pageType)
            """

        create_table_query_collect = f"""
    CREATE TABLE IF NOT EXISTS ozon_ads_collection_{self.add_name} (
        date Date,
        campaignId UInt64,
        collect Bool
    ) ENGINE = ReplacingMergeTree(collect)
    ORDER BY (campaignId, date)
    """

        now = datetime.now()
        yesterday = now - timedelta(days=1)
        self.client.command(create_table_query_campaigns)
        self.client.command(create_table_query_data_sp)
        self.client.command(create_table_query_data_sis)
        self.client.command(create_table_query_data_sku)
        self.client.command(create_table_query_data_shelf)
        self.client.command(create_table_query_data_banner)
        self.client.command(create_table_query_collect)
        token = self.get_token(self.clientid, self.token)

        names_code = self.get_names(token)


        self.client.command(optimize_campaigns)
        time.sleep(5)

        if names_code == 200:
            active_campaigns = self.get_campaigns_in_period(token, self.start)

        # забираем активные из ozoncampaigns
            active_campaigns_query = f"""
                    SELECT id, createdAt, toDate
                    FROM ozon_ads_campaigns_{self.add_name}
                    WHERE id IN ({', '.join(map(str, active_campaigns))})
                    """
            active_campaigns_query_result = self.client.query(active_campaigns_query)
            df_campaigns = pd.DataFrame(active_campaigns_query_result.result_rows, columns=['id', 'createdAt', 'toDate'])
            df_campaigns['createdAt'] = pd.to_datetime(df_campaigns['createdAt']).dt.date
            df_campaigns['toDate'] = pd.to_datetime(df_campaigns['toDate']).dt.date

        # формируем список заданий для ozoncollection
            campaigns_date_list = []
            yesterday_date = yesterday.strftime("%Y-%m-%d")
            for _, row in df_campaigns.iterrows():
                advertId = row['id']
                start_date = row['createdAt'].strftime('%Y-%m-%d')
                end_date = row['toDate'].strftime('%Y-%m-%d')
                if end_date > yesterday_date:
                    end_date = yesterday_date
                if start_date < self.start:
                    start_date = self.start
                date_list = self.create_date_list(start_date, end_date)
                for date in date_list:
                    campaigns_date_list.append((datetime.strptime(date, '%Y-%m-%d').date(), advertId, False))
            df_active_dates = pd.DataFrame(campaigns_date_list, columns=['date', 'campaignId', 'collect'])

        # вставляем задания в ozoncollection и делаем оптимайз
            self.ch_insert(df_active_dates, f'ozon_ads_collection_{self.add_name}')
            self.client.command(optimize_collection)
            time.sleep(10)

        # отбираем несделанные даты из ozoncollection
            false_dates_query = f"""
                    SELECT distinct date  
                    FROM ozon_ads_collection_{self.add_name}
                    WHERE collect = False"""
            collect_days_rows = self.client.query(false_dates_query).result_rows
            collect_days = [item[0] for item in collect_days_rows]
            n_days_ago = now - timedelta(days=self.backfill_days)

        # для каждой даты находим актуальный список кампаний на сбор
            for day in collect_days:
                if self.err429 == False:
                    token = self.get_token(self.clientid, self.token)
                    difference = n_days_ago.date() - day
                    sql_date = day.strftime('%Y-%m-%d')
                    false_campaigns_by_date_query = f"""
                            SELECT campaignId  
                            FROM ozon_ads_collection_{self.add_name}
                            WHERE collect = False AND date = '{sql_date}'"""
                    campaigns_to_collect_rows = self.client.query(false_campaigns_by_date_query).result_rows
                    campaigns_to_collect =  list(set([str(item[0]) for item in campaigns_to_collect_rows]))

                    # делаем сбор по чанкам для каждой даты
                    for chunk in self.chunk_list(campaigns_to_collect, 10):
                        body = []
                        success_list = []
                        for campaign in chunk:
                            body.append(campaign)
                            if difference.days >= 0:
                                success_list.append((day, int(campaign), True))
                        message = f'Платформа: OZON_ADS. Имя: {self.add_name}. Дата: {str(sql_date)}. Кампании: {str(body)}. Начало загрузки.'
                        self.common.log_func(self.bot_token, self.chat_list, message, 2)
            # получение данных и вставка в ozondata (единой транзакцией вместе с решением коллекшона)
                        try:
                            ozon_json = self.get_data(token, body, sql_date)
                            if int(ozon_json)==429:
                                self.err429 = True
                            df_success = pd.DataFrame(success_list, columns=['date', 'campaignId', 'collect'])
                            if int(ozon_json)==200:
                                self.ch_insert(df_success, f'ozon_ads_collection_{self.add_name}')
                                message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Дата: {str(sql_date)}. Кампании: {str(body)}. Результат: ОК."
                                self.common.log_func(self.bot_token, self.chat_list, message, 2)
                                self.client.command(optimize_collection)
                            if self.err429 == False:
                                time.sleep(2)
                        except Exception as e:
                            message = f"Платформа: OZON_ADS. Имя: {self.add_name}. Дата: {str(sql_date)}. Кампании: {str(body)}. Ошибка: {str(e)}."
                            self.common.log_func(self.bot_token, self.chat_list, message, 3)
                    if self.err429 == False:
                        time.sleep(10)

        self.client.command(optimize_data_sp)
        time.sleep(5)
        self.client.command(optimize_data_banner)
        time.sleep(5)
        self.client.command(optimize_data_shelf)
        time.sleep(5)
        self.client.command(optimize_data_sku)
        time.sleep(10)
        self.client.command(optimize_data_sis)
        time.sleep(5)
        self.client.command(optimize_campaigns)
        time.sleep(10)




from .common import Common
from .clickhouse import Clickhouse
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json


class BTRXbyDate:
    def __init__(self, bot_token:str, chats:str, message_type: str, subd: str,
                 host: str, port: str, username: str, password: str, database: str,
                 add_name: str, webhook_link: str ,  start: str, backfill_days: int, reports :str):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ', '').split(',')
        self.message_type = message_type
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.webhook_link = webhook_link
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.start = start
        self.reports = reports
        self.backfill_days = backfill_days
        self.platform = 'btrx'
        self.err429 = False
        self.source_dict = {
            'leads': {
                'platform': 'btrx',
                'report_name': 'leads',
                'upload_table': 'leads',
                'func_name': self.get_leads_create,
                'uniq_columns': 'ID',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 10
            },
                        'contacts': {
                'platform': 'btrx',
                'report_name': 'contacts',
                'upload_table': 'contacts',
                'func_name': self.get_contacts_create,
                'uniq_columns': 'ID,DATE_CREATE',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 10
            },
            'deals': {
                'platform': 'btrx',
                'report_name': 'deals',
                'upload_table': 'deals',
                'func_name': self.get_deals_create,
                'uniq_columns': 'ID,DATE_CREATE',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 10
            },
            'leads_changes': {
                'platform': 'btrx',
                'report_name': 'leads_changes',
                'upload_table': 'leads',
                'func_name': self.get_leads_modify,
                'uniq_columns': 'ID,DATE_CREATE',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 10
            },
            'contacts_changes': {
                'platform': 'btrx',
                'report_name': 'contacts_changes',
                'upload_table': 'contacts',
                'func_name': self.get_contacts_modify,
                'uniq_columns': 'ID,DATE_CREATE',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 10
            },
            'deals_changes': {
                'platform': 'btrx',
                'report_name': 'deals_changes',
                'upload_table': 'deals',
                'func_name': self.get_deals_modify,
                'uniq_columns': 'ID,DATE_CREATE',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 10
            },
        }

    def get_bitrix_data(self, report, filter_column, date1, date2):
        try:
            all_leads = []
            offset = 0
            id0 = '0'
            while True:
                print(f'offset: {str(offset)}')
                url = f'{self.webhook_link}/{report}?limit=50&start={offset}&filter[>{filter_column}]={date1}T00:00:00&filter[<{filter_column}]={date2}T23:59:59'
                response = requests.get(url)
                code =       response.status_code
                if code != 200:
                    response.raise_for_status()
                else:
                    data = response.json()['result']
                    id1 = str(data[0]['ID']).strip()
                    if id0 == id1:
                        break
                    all_leads += data
                if len(data) < 50:
                    break
                offset += 50
                time.sleep(3)
                id0 = id1
            return all_leads
        except Exception as e:
            message = f'Платформа: BTRX. Имя: {self.add_name}. Даты: {str(date1)}-{str(date2)}. Функция: get_bitrix_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def get_leads_create(self, date):
        try:
            final_result = self.get_bitrix_data('crm.lead.list.json', 'DATE_CREATE', date, date)
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_leads_create. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_leads_create. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_contacts_create(self, date):
        try:
            final_result = self.get_bitrix_data('crm.contact.list.json', 'DATE_CREATE', date, date)
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_contacts_create. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_contacts_create. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_deals_create(self, date):
        try:
            final_result = self.get_bitrix_data('crm.deal.list.json', 'DATE_CREATE', date, date)
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_deals_create. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_deals_create. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_leads_modify(self, date):
        try:
            final_result = self.get_bitrix_data('crm.lead.list.json', 'DATE_MODIFY', date, date)
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_leads_modify. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_leads_modify. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_contacts_modify(self, date):
        try:
            final_result = self.get_bitrix_data('crm.contact.list.json', 'DATE_MODIFY', date, date)
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_contacts_modify. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_contacts_modify. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_deals_modify(self, date):
        try:
            final_result = self.get_bitrix_data('crm.deal.list.json', 'DATE_MODIFY', date, date)
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_deals_modify. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: BTRX. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_deals_modify. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message



    # тип отчёта, дата -> данные в CH
    def collecting_manager(self):
        report_list = self.reports.replace(' ', '').lower().split(',')
        for report in report_list:
                self.clickhouse = Clickhouse(self.bot_token, self.chat_list, self.message_type, self.host, self.port, self.username, self.password,
                                             self.database, self.start, self.add_name, self.err429, self.backfill_days, self.platform)
                self.clickhouse.collecting_report(
                    self.source_dict[report]['platform'],
                    self.source_dict[report]['report_name'],
                    self.source_dict[report]['upload_table'],
                    self.source_dict[report]['func_name'],
                    self.source_dict[report]['uniq_columns'],
                    self.source_dict[report]['partitions'],
                    self.source_dict[report]['merge_type'],
                    self.source_dict[report]['refresh_type'],
                    self.source_dict[report]['history'],
                    self.source_dict[report]['frequency'],
                    self.source_dict[report]['delay']
                )












from .common import Common
from .clickhouse import Clickhouse
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json


class VKbyDate:
    def __init__(self, bot_token:str, chats:str, message_type: str, subd: str,
                 host: str, port: str, username: str, password: str, database: str,
                 add_name: str, token: str ,  start: str, backfill_days: int, reports :str):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ', '').split(',')
        self.message_type = message_type
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.token = token
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.start = start
        self.reports = reports
        self.backfill_days = backfill_days
        self.platform = 'vk'
        self.err429 = False
        self.source_dict = {
            'banners_stat': {
                'platform': 'vk',
                'report_name': 'banners_stat',
                'upload_table': 'banners_stat',
                'func_name': self.get_banners_stat,
                'uniq_columns': 'id,date',
                'partitions': 'date',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_date',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
                        'groups_stat': {
                'platform': 'vk',
                'report_name': 'groups_stat',
                'upload_table': 'groups_stat',
                'func_name': self.get_groups_stat,
                'uniq_columns': 'id,date',
                'partitions': 'date',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_date',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
            'campaigns_stat': {
                'platform': 'vk',
                'report_name': 'campaigns_stat',
                'upload_table': 'campaigns_stat',
                'func_name': self.get_campaigns_stat,
                'uniq_columns': 'id,date',
                'partitions': 'date',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_date',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
            'banners_list': {
                'platform': 'vk',
                'report_name': 'banners_list',
                'upload_table': 'banners_list',
                'func_name': self.get_banners_list,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
                        'groups_list': {
                'platform': 'vk',
                'report_name': 'groups_list',
                'upload_table': 'groups_list',
                'func_name': self.get_groups_list,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
            'campaigns_list': {
                'platform': 'vk',
                'report_name': 'campaigns_list',
                'upload_table': 'campaigns_list',
                'func_name': self.get_campaigns_list,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
        }

    def get_vk_statistics(self,endpoint, date1, date2, fields, offset):
        try:
            limit = 250
            base_url = f"https://ads.vk.com/api/v3/statistics/{endpoint}/day.json"
            params = {
                "date_from": date1,
                "date_to": date2,
                "fields": fields,
                "limit": limit,
                "offset": offset,
            }
            headers = {"Authorization": f"Bearer {self.token}"}
            response = requests.get(base_url, params=params, headers=headers)
            result = response.json()
            code = response.status_code
            if code != 200:
                response.raise_for_status()
            return result['items'], result['count']
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Даты: {str(date1)}-{str(date2)}. Функция: get_vk_statistics. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def fetch_vk_stat(self,endpoint, date):
        try:
            all_data = []
            offset = 0
            fields = 'all'
            for q in range(10000):
                data, count = self.get_vk_statistics(endpoint, date, date, fields, offset)
                all_data += data
                if int(count) < 250:
                    break
                else:
                    offset += 250
            for row in all_data:
                row['date']=date
            return self.common.spread_table(self.common.spread_table(self.common.spread_table(self.common.spread_table(all_data))))
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Даты: {str(date1)}-{str(date2)}. Функция: fetch_vk_stat. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def get_objects(self,report, offset):
        try:
            base_url = f"https://ads.vk.com/api/v2/{report}.json"
            params = {"limit": 50, "offset": offset}
            headers = {"Authorization": f"Bearer {self.token}"}
            response = requests.get(base_url, params=params, headers=headers)
            result = response.json()
            code = response.status_code
            if code != 200:
                response.raise_for_status()
            return result['items'], result['count']
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Функция: get_objects. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def fetch_vk_objects(self,report):
        try:
            all_objects = []
            offset = 0
            for q in range(10000):
                data, count = self.get_objects(report, offset)
                all_objects += data
                if int(count) < 50:
                    break
                else:
                    offset += 50
            return all_objects
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Функция: fetch_vk_objects. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def get_banners_stat(self, date):
        try:
            final_result = self.fetch_vk_stat('banners', date)
            message = f'Платформа: VK. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_banners_stat. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_banners_stat. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message

    def get_campaigns_stat(self, date):
        try:
            final_result = self.fetch_vk_stat('ad_plans', date)
            message = f'Платформа: VK. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_campaigns_stat. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_campaigns_stat. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message

    def get_groups_stat(self, date):
        try:
            final_result = self.fetch_vk_stat('ad_groups', date)
            message = f'Платформа: VK. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_groups_stat. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_groups_stat. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message

    def get_banners_list(self, date):
        try:
            final_result = self.fetch_vk_objects('banners')
            message = f'Платформа: VK. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_banners_list. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_banners_list. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message

    def get_groups_list(self, date):
        try:
            final_result = self.fetch_vk_objects('ad_groups')
            message = f'Платформа: VK. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_groups_list. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_groups_list. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message

    def get_campaigns_list(self, date):
        try:
            final_result = self.fetch_vk_objects('ad_plans')
            message = f'Платформа: VK. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_campaigns_list. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: VK. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_campaigns_list. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message

    # тип отчёта, дата -> данные в CH
    def collecting_manager(self):
        report_list = self.reports.replace(' ', '').lower().split(',')
        for report in report_list:
                self.clickhouse = Clickhouse(self.bot_token, self.chat_list, self.message_type, self.host, self.port, self.username, self.password,
                                             self.database, self.start, self.add_name, self.err429, self.backfill_days, self.platform)
                self.clickhouse.collecting_report(
                    self.source_dict[report]['platform'],
                    self.source_dict[report]['report_name'],
                    self.source_dict[report]['upload_table'],
                    self.source_dict[report]['func_name'],
                    self.source_dict[report]['uniq_columns'],
                    self.source_dict[report]['partitions'],
                    self.source_dict[report]['merge_type'],
                    self.source_dict[report]['refresh_type'],
                    self.source_dict[report]['history'],
                    self.source_dict[report]['frequency'],
                    self.source_dict[report]['delay']
                )












from .wb_by_date import WBbyDate
from .yd_by_date import YDbyDate
from .ozon_by_date import OZONbyDate
from .market_by_date import MRKTbyDate
from .getcourse_by_date import GCbyDate
from .ym_by_date import YMbyDate
from .btrx_by_date import BTRXbyDate
from .vk_by_date import VKbyDate
from .alfacrm_by_date import ALFAbyDate


import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import chardet
import json
import math
from transliterate import translit

class Common:
    def __init__(self, bot_token:str, chat_list:str, message_type: str):
        self.bot_token = bot_token
        self.chat_list = chat_list

        if message_type == 'all':
            self.value = 1
        elif message_type == 'key':
            self.value = 2
        else:
            self.value = 3
        self.now = datetime.now()
        self.today = datetime.now().date()

    def log_func(self, bot_token, chat_ids,message, value):
        try:
            print(message)
            log_file_path = "/app/log.txt"
            os.makedirs(os.path.dirname(log_file_path), exist_ok=True)
            if value >= self.value:
                with open(log_file_path, "a") as log_file:
                    log_file.write(message + "\n\n")
            self.send_logs_clear(bot_token, chat_ids, message)
        except Exception as e:
            print(f'Ошибка log_func: {e}')

    def send_logs_clear(self,bot_token, chat_ids, message):
        try:
            log_file_path = "/app/log.txt"
            if not os.path.exists(log_file_path):
                print("Файл лога не существует.")
            with open(log_file_path, "r") as log_file:
                content = log_file.read()
            if len(content) > 1000:
                self.message_text = content
                self.send_logs(bot_token, chat_ids)
                with open(log_file_path, "w") as log_file:
                    log_file.write("")  # Очищаем файл
                print("Файл очищен, длина содержимого превышала 1000 символов.")
            return content
        except Exception as e:
            print(f'Ошибка send_logs_clear: {e}')

    def send_logs(self, bot_token, chat_ids):
        try:
            url = f'https://api.telegram.org/bot{bot_token}/sendMessage'
            for chat_id in chat_ids:
                try:
                    params = {'chat_id': chat_id, 'text': self.message_text}
                    response = requests.get(url, params=params)
                    if response.status_code != 200:
                        print(f"Ошибка отправки сообщения в чат {chat_id}: {response.text}")
                except:
                    print(f"Ошибка отправки сообщения в чат {chat_id}: {response.text}")
            self.message_text = ''
        except Exception as e:
            print(f'Ошибка send_logs: {e}')

    def is_empty(self, result):
        if not result:
            return True
        if isinstance(result, list) and all(isinstance(item, dict) and not item for item in result):
            return True
        return False

    def flip_date(self, date_text):
        try:
            year, month, day = date_text.split('-')
            return f"{day}.{month}.{year}"
        except Exception as e:
            print(f'Ошибка flip_date: {e}')

    def is_error(self, result):
        if isinstance(result, str):
            if 'Ошибка:' in result:
                return True
        return False

    def shorten_text(self, text):
        # Используем хеш-функцию md5 для сокращения строки
        hash_object = hashlib.md5(text.encode())  # Можно также использовать sha256
        return hash_object.hexdigest()[:10]  # Возвращаем первые 10 символов хеша

    def transliterate_key(self, key):
        tr = translit(key, 'ru', reversed=True)
        tr = tr.strip().replace(' ', '_').replace('-', '_').replace(",", '').replace("'", '').replace(".", '').replace("(",'').replace(")", '').lower()
        return  tr

    def replace_keys_in_data(self, dictionaries_list):
        updated_list = []
        for dictionary in dictionaries_list:
            updated_dict = {}
            for key, value in dictionary.items():
                # Заменяем дефисы на подчеркивания в ключе
                new_key = key.replace('-', '_')
                # Добавляем новый ключ и значение в обновленный словарь
                updated_dict[new_key] = value
            updated_list.append(updated_dict)
        return updated_list

    def shift_date(self, date_str, days=7):
        # Преобразуем строку в объект datetime
        date_obj = datetime.strptime(date_str, '%Y-%m-%d')
        # Сдвигаем дату на указанное количество дней назад
        new_date = date_obj - timedelta(days=days)
        # Преобразуем дату обратно в строку
        return new_date.strftime('%Y-%m-%d')


    def keep_last_20000_lines(self,file_path):
        with open(file_path, 'rb') as file:
            raw_data = file.read()
            result = chardet.detect(raw_data)
            encoding = result['encoding']
        with open(file_path, 'r', encoding=encoding) as file:
            lines = file.readlines()
        last_20000_lines = lines[-20000:]
        with open(file_path, 'w', encoding=encoding) as file:
            file.writelines(last_20000_lines)

    # значение -> тип значения для clickhouse
    def get_data_type(self, column, getvalue, partitions):
        getvalue = str(getvalue)
        part_list = partitions.replace(' ', '').split(',')
        if getvalue == None or getvalue == 'None' or getvalue.strip() == '': return 'None'
        if getvalue.lower() == 'false' or getvalue.lower() == 'true':
            return 'UInt8'
        date_formats = [
            "%Y-%m-%dT%H:%M:%S.%f%z",  # 2023-10-22T16:36:15.507+0000
            "%Y-%m-%d %H:%M:%S.%f%z",  # 2023-10-22 16:36:15.507+0000
            "%Y-%m-%dT%H:%M:%S%z",  # 2023-10-22T16:36:15+0000
            "%Y-%m-%d %H:%M:%S%z",  # 2023-10-22 16:36:15+0000
            "%Y-%m-%dT%H:%M:%S.%f",  # 2023-10-22T16:36:15.507 (без таймзоны)
            "%Y-%m-%d %H:%M:%S.%f",  # 2023-10-22 16:36:15.507 (без 'T')
            "%Y-%m-%dT%H:%M:%S",  # 2023-10-22T16:36:15 (без миллисекунд и таймзоны)
            "%Y-%m-%d %H:%M:%S",  # 2023-10-22 16:36:15 (без 'T', без миллисекунд)
            "%Y-%m-%d",  # 2023-10-22 (только дата)
            "%Y.%m.%d",  # 2023-10-22 (только дата)
            "%d.%m.%Y",  # 22-10-2023 (европейский формат)  # Формат Date с днем в начале: 08-09-2021
            "%d-%m-%Y",  # 22-10-2023 (европейский формат)  # Формат Date с днем в начале: 08-09-2021
            '%Y/%m/%d',  # Формат Date через слэш: 2024/09/01
            '%H:%M:%S',  # Формат Time: 21:20:10
        ]
        for date_format in date_formats:
            try:
                parsed_date = datetime.strptime(getvalue.replace('Z', ''), date_format)
                # Если дата меньше 1970 года — это не допустимая дата для ClickHouse
                if parsed_date.year < 1970:
                    return 'String'
                # Определяем тип на основе формата
                if date_format in ['%Y-%m-%d', '%d-%m-%Y','%Y.%m.%d', '%d.%m.%Y', '%Y/%m/%d']:
                    return 'Date'  # Это формат Date
                elif date_format == '%H:%M:%S':
                    return 'Time'  # Это формат Time
                else:
                    return 'DateTime'  # Форматы с датой и временем
            except ValueError:
                continue
        try:
            float_value = float(getvalue)
            if len(str(float_value)) < 15 and column not in part_list:
                return 'Float64'
        except:
            pass
        return 'String'


    def column_to_datetime(self, date_str):
        if pd.isna(date_str):
            return None
        date_str = date_str.strip()

        # Обрабатываем таймзону 'Z' (UTC) и заменяем на '+0000'
        if date_str.endswith('Z'):
            date_str = date_str[:-1] + '+0000'
        # Обрабатываем таймзоны вида +00:00 и заменяем на +0000
        elif '+' in date_str and date_str.endswith(':00'):
            date_str = date_str[:-3] + date_str[-2:]

        date_formats = [
            "%Y-%m-%dT%H:%M:%S.%f%z",  # 2023-10-22T16:36:15.507+0000
            "%Y-%m-%d %H:%M:%S.%f%z",  # 2023-10-22 16:36:15.507+0000
            "%Y-%m-%dT%H:%M:%S%z",  # 2023-10-22T16:36:15+0000
            "%Y-%m-%d %H:%M:%S%z",  # 2023-10-22 16:36:15+0000
            "%Y-%m-%dT%H:%M:%S.%f",  # 2023-10-22T16:36:15.507 (без таймзоны)
            "%Y-%m-%d %H:%M:%S.%f",  # 2023-10-22 16:36:15.507 (без 'T')
            "%Y-%m-%dT%H:%M:%S",  # 2023-10-22T16:36:15 (без миллисекунд и таймзоны)
            "%Y-%m-%d %H:%M:%S",  # 2023-10-22 16:36:15 (без 'T', без миллисекунд)
            "%Y-%m-%d",  # 2023-10-22 (только дата)
            "%d-%m-%Y",  # 22-10-2023 (европейский формат)
            "%Y.%m.%d",  # 2023-10-22 (только дата)
            "%d.%m.%Y"  # 22-10-2023 (европейский формат)
        ]

        for fmt in date_formats:
            try:
                dt = datetime.strptime(date_str, fmt)
                return dt.strftime("%Y-%m-%d %H:%M:%S.%f")
            except ValueError:
                continue
        return None
        # список словарей (данные) -> список поле_типданных

    def analyze_column_types(self, data, uniq_columns, partitions, text_columns_set):
        try:
            null_columns = []
            column_types = {}
            # Проходим по всем строкам в данных
            for row in data:
                for column, anvalue in row.items():
                    value_type = self.get_data_type(column, anvalue, partitions)  # Определяем тип данных
                    if column not in column_types:
                        column_types[column] = set()  # Создаем множество для уникальных типов
                    column_types[column].add(value_type)
            # Приводим типы столбцов к общему типу
            final_column_types = {}
            for column, types in column_types.items():
                try: types.remove('None')
                except: pass
                if len(types) == 1 and column.strip() not in  text_columns_set:
                    final_column_types[column] = next(iter(types))
                elif len(types) == 0:
                    final_column_types[column] = 'None'
                else:
                    final_column_types[column] = 'String'  # Если разные типы, делаем строкой
            create_table_query = []
            # non_nullable_list = uniq_columns.replace(' ','').split(',')+[partitions.strip()]
            for field, data_type in final_column_types.items():
                create_table_query.append(f"{field} {data_type}")
        except Exception as e:
            message = f'Функция: analyze_column_types. Ошибка: {e}'
            self.log_func(self.bot_token, self.chat_list,message, 3)


        return create_table_query

    # список словарей (данные) -> датафрейм с нужными типами
    def check_and_convert_types(self, data, uniq_columns, partitions, text_columns_set):
        try:
            columns_list=self.analyze_column_types(data, uniq_columns, partitions,text_columns_set)
            df=pd.DataFrame(data,dtype=str)
            type_mapping = {
                'UInt8': 'bool',
                'Date': 'datetime64[ns]',  # pandas формат для дат
                'DateTime': 'datetime64[ns]',  # pandas формат для дат с временем
                'String': 'object',  # Строковый формат в pandas
                'Float64': 'float64',  # float64 тип в pandas
            }
            for item in columns_list:
                column_name, expected_type = item.split()  # Разделяем по пробелу: 'column_name expected_type'
                if column_name in df.columns:
                    expected_type = expected_type.strip()
                    try:
                        if expected_type in ['Date']:
                            df[column_name] = df[column_name].apply(self.column_to_datetime)
                            df[column_name] = pd.to_datetime(df[column_name], errors='raise').dt.date
                            df[column_name] = df[column_name].fillna(pd.to_datetime('1970-01-01').date())
                        if expected_type in ['DateTime']:
                            df[column_name] = df[column_name].apply(self.column_to_datetime)
                            df[column_name] = pd.to_datetime(df[column_name], errors='raise')
                            df[column_name] = df[column_name].fillna(pd.Timestamp('1970-01-01'))
                        elif expected_type in ['UInt8']:
                            df[column_name] = df[column_name].replace({'True': True, 'False': False, 'true': True, 'false': False, })
                            df[column_name] = df[column_name].fillna(False)
                            df[column_name] = df[column_name].astype('bool')
                        elif expected_type in ['Float64']:
                            df[column_name] = pd.to_numeric(df[column_name], errors='raise').astype('float64')
                            df[column_name] = df[column_name].fillna(0)
                        elif expected_type in ['String']:
                            df[column_name] = df[column_name].astype(str)
                            df[column_name] = df[column_name].fillna("")
                        elif 'None' in expected_type:
                            df = df.drop(columns=[column_name])
                    except Exception as e:
                        message = f"Функция: check_and_convert_types. Ошибка при преобразовании столбца '{column_name}': {e}"
                        self.log_func(self.bot_token, self.chat_list, message, 3)
            df['timeStamp'] = self.now
            message = f'Функция: check_and_convert_types. Датафрейм успешно преобразован'
            self.log_func(self.bot_token, self.chat_list, message, 2)
        except Exception as e:
            message= f'Функция: check_and_convert_types. Ошибка преобразования df: {e}'
            self.log_func(self.bot_token, self.chat_list, message, 3)
        return df

    def to_collect(self, schedule_str, date_str):
        try:
            today = datetime.strptime(date_str, '%Y-%m-%d').date()
        except ValueError:
            raise ValueError("Дата должна быть в формате 'YYYY-MM-DD'")
        day_of_week = today.strftime('%A').lower()  # День недели (например, 'friday')
        day_of_month = today.day  # Число месяца (например, 22)
        schedule_list = [s.strip().lower() for s in schedule_str.split(',')]
        for schedule in schedule_list:
            if schedule == 'daily':  # Если указано "daily", всегда возвращаем True
                return True
            if schedule == day_of_week:  # Проверка дня недели (например, 'friday')
                return True
            if schedule.isdigit() and int(schedule) == day_of_month:  # Проверка числа месяца
                return True
        return False

    def spread_table(self, source_list):
        result_list = []
        for row in source_list:
            row_dict = {}
            for key, value in row.items():
                if isinstance(value, dict):
                    for name, inner_value in dict(value).items():
                        row_dict[f'{key}_{name}'] = inner_value
                else:
                    row_dict[f'{key}'] = value
            result_list.append(row_dict)
        return result_list



from setuptools import setup, find_packages

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='morin',
    version='0.3.47',
    packages=find_packages(),
    description='Помощь в подключениях и загрузке в БД',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Александр Морин',
    author_email='y.director@yandex.ru',
    url='https://github.com/morinad/morin',
    install_requires=['requests', 'clickhouse_connect', 'pandas', 'python-dateutil'],
)